.\" file MASS/cov.rob.d
.\" copyright (C) 1998,9 W. N. Venables and B. D. Ripley
.\"
.BG
.FN cov.rob
.TL
Robust Estimation of Multivariate Location and Scatter
.DN
Compute a multivariate location and scale estimate with a high
breakdown point \(en this can be thought of as estimating the mean and
covariance of the `good' part of the data.
.CS
cov.rob(x, cor=FALSE, quantile.used=floor((n + p + 1)/2),
        method=c("mve", "mcd", "classical"), nsamp="best", seed)
.RA
.AG x
a matrix or data frame.
.AG cor
should the returned result include a correlation matrix?
.AG quantile.used
the minimum number of the data points regarded as `good' points.
.AG method
the method to be used \(en minimum volume ellipsoid, minimum
covariance determinant or classical product-moment.
.AG nsamp
the number of samples or `"best"' or `"exact"' or `"sample"'. If
`"sample"' the number chosen is `min(5*p, 3000)', taken from Rousseeuw
and Hubert (1997). If `"best"' exhaustive enumeration is done up to
5000 samples: if `"exact"' exhaustive enumeration will be attempted
however many samples are needed.
.AG seed
the seed to be used for random sampling: see `.Random.seed'. The
current value of `.Random.seed' will be preserved if it is set..
.RT
A list with components
.RC center
the final estimate of location.
.RC cov
the final estimate of scatter.
.RC cor
(only is `cor=T') the estimate of the correlation
matrix.
.RC sing
message giving number of singular samples out of total
.RC crit
the value of the criterion on log scale. For MCD this is
the determinant, and for MVE it is proportional to the volume.
.RC best
the subset used. For MVE the best sample, for MCD the best
set of size `quantile.used'.
.RC final
the final subset of `good' points.
.RC n.obs
total number of observations.
.DT
For method `"mve"', an approximate search is made of a subset of
size `quantile.used' with an enclosing ellipsoid of smallest volume; in
method `"mcd"' it is the volume of the Gaussian confidence
ellipsoid, equivalently the determinant of the classical covariance
matrix, that is minimized. The mean of the subset provides a first
estimate of the location, and the rescaled covariance matrix a first
estimate of scatter. The Mahalanobis distances of all the points from
the location estimate for this covariance matrix are calculated, and
those points within the 97.5% point under Gaussian assumptions are
declared to be `good'. The final estimates are the mean and rescaled
covariance of the `good' points.

The rescaling is by the appropriate percentile under Gaussian data; in
addition the first covariance matrix has an \fIad hoc\fP finite-sample
correction given by Marazzi.

For method `"mve"' the search is made over ellipsoids determined
by the covariance matrix of `p' of the data points. For method
`"mcd"' an additional improvement step suggested by Rousseeuw and
van Driessen (1997) is used, in which once a subset of size
`quantile.used' is selected, an ellipsoid based on its covariance
is tested (as this will have no larger a determinant, and may be smaller).

.SH REFERENCES
P. J. Rousseeuw and A. M. Leroy (1987) 
.ul
Robust Regression and Outlier Detection.
Wiley.
.PP
A. Marazzi (1993) 
.ul
Algorithms, Routines and S Functions for Robust Statistics.
Wadsworth & Brooks/Cole.
.PP 
P. J. Rousseeuw and B. C. van Zomeren (1990) Unmasking
multivariate outliers and leverage points, 
.ul
Journal of the American Statistical Association, 
\fB85\fP, 633-639.
.PP 
P. J. Rousseeuw and K. van Driessen (1999) A fast algorithm for the
minimum covariance determinant estimator.
.ul
Technometrics
\fB41\fP, 212-223.
.SA
`lqs'
.EX
stackloss <- data.frame(stack.x, stack.loss)
cov.rob(stackloss)
cov.rob(stack.x, method="mcd", nsamp="exact")
.KW robust
.KW multivariate
.WR
