.\" file MASS/cov.trob.d
.\" copyright (C) 1997-9 W. N. Venables and B. D. Ripley
.\"
.BG
.FN cov.trob
.TL
Covariance Estimation for Multivariate t Distribution
.DN
Estimates a covariance or correlation matrix assuming the data came
from a multivariate t distribution: this provides some degree of
robustness to outlier without giving a high breakdown point.
.CS
cov.trob(x, wt=rep(1, n), cor=F, center=T, nu=5, maxit=25, tol=0.01)
.RA
.AG x
data  matrix. Missing values (NAs) are not allowed.
.OA
.AG wt
A vector of weights for each case: these are treated as if the case `i'
actually occurred `wt[i]' times.
.AG cor
Flag to choose between returning the correlation (`cor=T') or
covariance (`cor=F') matrix.
.AG center
a logical value or a numeric vector providing the location about which
the covariance is to be taken. If `center=F', no centering is done;
if `center=T' the MLE of the location vector is used.
.AG nu
``degrees of freedom'' for the multivariate t distribution. Must exceed
2 (so that the covariance matrix is finite).
.AG maxit
Maximum number of iterations in fitting.
.AG tol
Convergence tolerance for fitting.
.RT
A list with the following components
.RC cov
the fitted covariance matrix.
.RC center
the estimated or specified location vector.
.RC wt
the specified weights: only returned if the `wt' argument was given.
.RC n.obs
the number of cases used in the fitting.
.RC cor
the fitted correlation matrix: only returned if `cor = T'.
.RC call
The matched call.
.RC iter
The number of iterations used.
.SH REFERENCE
J. T. Kent and D. E. Tyler and Y. Vardi (1994)
A curious likelihood identity for the multivariate t-distribution.
.ul
Communications in Statistics---Simulation and Computation,
\fB23\fP, 441-453.
.SA
`cov', `cov.wt', `cov.mve'
.EX
stackloss <- data.frame(stack.x, stack.loss)
cov.trob(stackloss)
.KW multivariate
.WR
