.\" file MASS/glm.nb.d
.\" copyright (C) 1994-9 W. N. Venables and B. D. Ripley
.\"
.BG
.FN glm.nb
.TL
Fit a Negative Binomial Generalized Linear Model
.DN
A modification of the system function `glm()' to include estimation of the
additional parameter, `theta', for a Negative Binomial generalized linear
model.
.CS
glm.nb(formula, ...,  init.theta, link=log)
.RA
.AG formula
Model formula, as for `glm()'.
.OA
.AG ...
Any other arguments for the `glm()' function except `family'.
.AG init.theta
Optional initial value for the theta parameter.  If omitted a moment
estimator after an initial fit using a Poisson GLM is used.
.AG link
The link function.  Currently must be one of `log', `sqrt' or `identity'.
.RT
A fitted model object of class `negbin' inheriting from `glm' and `lm'.
The object is like the output of `glm' but contains three additional
components, namely `theta' for the ML estimate of theta, `SE.theta' for
its approximate standard error (using observed rather than expected
information), and `twologlik' for twice the log-likelihood function.
.DT
An alternating iteration process is used.  For given `theta' the GLM
is fitted using the same process as used by `glm()'.  For fixed means
the `theta' parameter is estimated using score and information
iterations.  The two are alternated until convergence of both. (The
number of alternations and the number of iterations when estimating
`theta' are controlled by the `maxit' parameter of `glm.control'.)
.PP
Setting `trace > 0' traces the alternating iteration process. Setting
`trace > 1' traces the `glm' fit, and setting `trace > 2' traces the
estimation of `theta'.
.SA
`glm', `negative.binomial', `anova.negbin', `summary.negbin',
`theta.ml', `theta.md', `theta.mm'
.EX
quine.nb1 <- glm.nb(Days ~ Sex/(Age + Eth*Lrn), data=quine)
quine.nb2 <- update(quine.nb1, . ~ . + Sex:Age:Lrn)
quine.nb3 <- update(quine.nb2, Days ~ .^4)
anova(quine.nb1, quine.nb2, quine.nb3)
.KW regression
.KW models
.WR
