.\" file MASS/sammon.d
.\" copyright (C) 1994-9 W. N. Venables and B. D. Ripley
.\"
.BG
.FN sammon
.TL
Sammon's Non-Linear Mapping
.DN
One form of non-metric multidimensional scaling.
.CS
sammon(d, y = cmdscale(d, k), k = 2, niter = 100, trace = T, 
       magic = 0.2, tol = 1e-4)
.RA
.AG d
distance structure of the form returned by `dist', or a full, symmetric
matrix.  Data are assumed to be dissimilarities or relative distances,
but must be positive except for self-distance.
.OA
.AG y
An initial configuration. If none is supplied, `cmdscale' is used to provide
the classical solution.
.AG k
The dimension of the configuration.
.AG niter
The maximum number of iterations.
.AG trace
Logical for tracing optimization. Default `T'.
.AG magic
initial value of the step size constant in diagonal Newton method.
.AG tol
Tolerance for stopping, in units of stress.
.RT
Two components:
.RC points
A two-column vector of the fitted configuration.
.RC stress
The final stress achieved.
.SE
If trace is true, the initial stress and the current stress are printed
out every 10 iterations.
.DT
This chooses a two-dimensional configuration to minimize the stress,
the sum of squared differences between the input distances and those
of the configuration, weighted by the distances, the whole sum being
divided by the sum of input distances to make the stress scale-free.
.PP
An iterative algorithm is used, which will usually converge in around
50 iterations. As this is necessarily an O(n^2) calculation, it is slow
for large datasets. Further, since the configuration is only determined
up to rotations and reflections (by convention the centroid is at the
origin), the result can vary considerably from machine to machine.
In this release the algorithm has been modified by adding a step-length
search (`magic') to ensure that it always goes downhill.
.SH REFERENCE
Sammon, J. W. (1969) 
A non-linear mapping for data structure analysis.
.ul
IEEE Trans. Comput.
\fBC-18\fR 401-409.
.SA
`cmdscale', `isoMDS'
.EX
swiss.sam <- sammon(dist(swiss.x))
plot(swiss.sam$points, type="n")
text(swiss.sam$points, labels=as.character(1:nrow(swiss.x)))
.KW multivariate
.WR
